/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.bpmn2designer;

import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;
import org.ow2.orchestra.designer.bpmn.model.PoolModel;
import org.ow2.orchestra.designer.bpmn.model.ProcessModel;
import org.ow2.orchestra.designer.bpmn.model.operations.InterfaceModel;
import org.ow2.orchestra.designer.bpmn.model.operations.OperationModel;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.Definitions;
import org.ow2.orchestra.jaxb.bpmn.Operation;
import org.ow2.orchestra.jaxb.bpmn.Participant;
import org.ow2.orchestra.jaxb.bpmn.TCollaboration;
import org.ow2.orchestra.jaxb.bpmn.TFlowElement;
import org.ow2.orchestra.jaxb.bpmn.TInterface;
import org.ow2.orchestra.jaxb.bpmn.TItemDefinition;
import org.ow2.orchestra.jaxb.bpmn.TMessage;
import org.ow2.orchestra.jaxb.bpmn.TProcess;
import org.ow2.orchestra.jaxb.bpmn.TRootElement;
import org.ow2.orchestra.jaxb.bpmn.bpmndi.BPMNDiagram;

/**
 * @author Loic Albertin
 */
public final class Bpmn2DesignerTransformer {

  /**
   * Transforms a BPMN {@link Definitions} into a designer {@link ProcessModel}
   *
   * @param definitions The {@link Definitions} to modelToBpmn
   * @return Returns a corresponding {@link ProcessModel}
   * @throws TransformationException
   */
  public static ProcessModel bpmnToModel(final Definitions definitions) throws TransformationException {
    final Bpmn2DesignerContext context = new Bpmn2DesignerContext();
    context.setDefinitions(definitions);
    final ProcessModel processModel = new ProcessModel();
    context.setProcessModel(processModel);

    // Initiate the context with diagram elements
    for (final BPMNDiagram diagram : definitions.getBPMNDiagrams()) {
      context.initiateDiagramElementMap(diagram.getBPMNPlane());
    }

    // Now lets see root elements (mainly the process for now)
    for (final JAXBElement<? extends TRootElement> rootElement : definitions.getRootElements()) {
      if (rootElement.getDeclaredType().equals(TProcess.class)) {
        final TProcess tProcess = (TProcess) rootElement.getValue();
        context.initiateFlowElementMap(tProcess);

        processModel.setName(tProcess.getName());
        processModel.setId(tProcess.getId());

        Bpmn2DesignerTransformer.bpmnToModelProcessInterfaces(context);

        for (final JAXBElement<? extends TFlowElement> flowElement : tProcess.getFlowElements()) {
          org.ow2.orchestra.designer.models.transformers.bpmn2designer.ElementTransformer.bpmnToModel(flowElement.getValue(), context);
        }
      }
    }
    return processModel;
  }

  private static void bpmnToModelProcessInterfaces(final Bpmn2DesignerContext context) throws TransformationException {
    final ProcessModel processModel = context.getProcessModel();
    final QName processQName = new QName(context.getDefinitions().getTargetNamespace(), processModel.getId());

    for (final JAXBElement<? extends TRootElement> rootElement : context.getDefinitions().getRootElements()) {
      if (rootElement.getDeclaredType().equals(TCollaboration.class)) {
        final TCollaboration collaboration = (TCollaboration) rootElement.getValue();
        for (final Participant participant : collaboration.getParticipants()) {
          if (processQName.equals(participant.getProcessRef())) {
            // find process interfaces
            generatePoolModelInterfaces(context, processModel, participant);
            // Now retrieve positions
            ShapeTransformer.bpmnToModel(participant.getId(), processModel, context);
          } else {
            PoolTransformer.bpmnToModel(participant, context);
          }
        }
      }
    }
  }

  public static void generatePoolModelInterfaces(Bpmn2DesignerContext context, PoolModel poolModel, Participant participant) {
    final List<TInterface> poolInterfaces = new ArrayList<TInterface>();
    for (final JAXBElement<? extends TRootElement> rootElement : context.getDefinitions().getRootElements()) {
      if (rootElement.getDeclaredType().equals(TInterface.class)) {
        final TInterface tInterface = (TInterface) rootElement.getValue();
        final QName tInterfaceQName = new QName(context.getDefinitions().getTargetNamespace(), tInterface.getId());
        if (participant.getInterfaceReves().contains(tInterfaceQName)) {
          poolInterfaces.add(tInterface);
        }
      }
    }
    // create interface models
    for (final TInterface poolInterface : poolInterfaces) {
      final InterfaceModel interfaceModel = new InterfaceModel();
      interfaceModel.setName(poolInterface.getName());
      interfaceModel.setId(poolInterface.getId());
      interfaceModel.setPoolModel(poolModel);
      poolModel.getInterfaces().put(interfaceModel.getName(), interfaceModel);
      if (poolInterface.getImplementationRef() != null) {
        interfaceModel.setPortTypeQName(poolInterface.getImplementationRef().toString());
      }
      // add interface operations
      for (final Operation operation : poolInterface.getOperations()) {
        final OperationModel operationModel = new OperationModel();
        operationModel.setName(operation.getName());
        operationModel.setId(operation.getId());
        operationModel.setInterfaceModel(interfaceModel);
        interfaceModel.getOperations().put(operationModel.getName(), operationModel);
        TMessage inMessage = null;
        TMessage outMessage = null;
        for (final JAXBElement<? extends TRootElement> rootElement : context.getDefinitions().getRootElements()) {
          if (rootElement.getValue() instanceof TMessage) {
            if (((TMessage) rootElement.getValue()).getId().equals(operation.getInMessageRef().getLocalPart())) {
              inMessage = (TMessage) rootElement.getValue();
            }
            if (operation.getOutMessageRef() != null &&
                ((TMessage) rootElement.getValue()).getId().equals(operation.getOutMessageRef().getLocalPart())) {
              outMessage = (TMessage) rootElement.getValue();
            }
          }
        }
        for (final JAXBElement<? extends TRootElement> rootElement : context.getDefinitions().getRootElements()) {
          if (rootElement.getValue() instanceof TItemDefinition) {
            if (((TItemDefinition) rootElement.getValue()).getId().equals(inMessage.getItemRef().getLocalPart())) {
              operationModel.setInMessageType(((TItemDefinition) rootElement.getValue()).getStructureRef().toString());
            }
            if (outMessage != null &&
                ((TItemDefinition) rootElement.getValue()).getId().equals(outMessage.getItemRef().getLocalPart())) {
              operationModel.setOutMessageType(((TItemDefinition) rootElement.getValue()).getStructureRef().toString());
            }
          }
        }
      }
    }
  }
}
