/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.designer2bpmn;

import java.util.List;
import javax.xml.namespace.QName;
import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.EventModel;
import org.ow2.orchestra.designer.bpmn.model.GatewayModel;
import org.ow2.orchestra.designer.bpmn.model.PoolModel;
import org.ow2.orchestra.designer.bpmn.model.SequenceFlowModel;
import org.ow2.orchestra.designer.bpmn.model.TaskModel;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.TFlowElement;
import org.ow2.orchestra.jaxb.bpmn.TFlowNode;

/**
 * Utility class used to retrieve the correct transformer for a given object
 *
 * @author Loic Albertin
 */
public final class ElementTransformer {

  /**
   * Utility classes should not have public or default constructors.
   */
  private ElementTransformer() {
  }

  /**
   * Transform an {@link AbstractElement} into a {@link TFlowElement}. this
   * method mainly retrieve the correct transformer and call its modelToBpmn
   * method.
   *
   * @param element the element to transform
   * @param context the transformation context
   * @throws TransformationException Throws {@link TransformationException} if
   *                                 the given {@link AbstractElement} is not supported or if any of
   *                                 other sub-transformers throws a {@link TransformationException}
   */
  public static void modelToBpmn(final AbstractElement element, final Designer2BpmnContext context) throws TransformationException {
    if (!context.getModelsMap().containsKey(element)) {
      if (element instanceof GatewayModel) {
        final GatewayModel gatewayModel = (GatewayModel) element;
        GatewayTransformer.modelToBpmn(gatewayModel, context);
      } else if (element instanceof SequenceFlowModel) {
        final SequenceFlowModel seqFlowModel = (SequenceFlowModel) element;
        SequenceFlowTransformer.modelToBpmn(seqFlowModel, context);
      } else if (element instanceof EventModel) {
        final EventModel eventModel = (EventModel) element;
        EventTransformer.modelToBpmn(eventModel, context);
      } else if (element instanceof TaskModel) {
        final TaskModel taskModel = (TaskModel) element;
        TaskTransformer.modelToBpmn(taskModel, context);
      } else if (element instanceof PoolModel) {
        // Transformation already done, ignore
      } else {
        // Designer model represent a sub-part of BPMN model so
        // any of its elements must have a transformation into BPMN.
        throw new TransformationException("BPMN conversion error: unsupported model element " + element.getClass().getSimpleName());
      }

      if (element instanceof AbstractElementWithPosition) {
        ElementTransformer.modelToBpmnIncomingOutgoingSequenceFlows((AbstractElementWithPosition) element, context);
      }
    }
  }

  public static void modelToBpmnIncomingOutgoingSequenceFlows(final AbstractElementWithPosition modelElement, final Designer2BpmnContext context) throws TransformationException {
    final TFlowElement contextElement = context.getModelsMap().get(modelElement);
    if (contextElement instanceof TFlowNode) {
      final TFlowNode bpmnElement = (TFlowNode) contextElement;
      final List<SequenceFlowModel> incomingSequenceFlows = modelElement.getIncomingSequenceFlows();
      if (incomingSequenceFlows != null) {
        for (final SequenceFlowModel incomingSequenceFlow : incomingSequenceFlows) {
          final QName incomingSequenceFlowQName = new QName(context.getDefinitions().getTargetNamespace(), incomingSequenceFlow.getId());
          if (!bpmnElement.getIncomings().contains(incomingSequenceFlowQName)) {
            bpmnElement.getIncomings().add(incomingSequenceFlowQName);
          }
        }
      }
      final List<SequenceFlowModel> outgoingSequenceFlows = modelElement.getOutgoingSequenceFlows();
      if (outgoingSequenceFlows != null) {
        for (final SequenceFlowModel outgoingSequenceFlow : outgoingSequenceFlows) {
          final QName outgoingSequenceFlowQName = new QName(context.getDefinitions().getTargetNamespace(), outgoingSequenceFlow.getId());
          if (!bpmnElement.getOutgoings().contains(outgoingSequenceFlowQName)) {
            bpmnElement.getOutgoings().add(outgoingSequenceFlowQName);
          }
        }
      }
    }
  }
}
